/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Create the ZxMig object.
 * @constructor
 */
function ZxMig() {
  ZxModule.call(this, ZxMig.MOD_ID, 'ZxMig', 'ZxMig');

  this.registerLoadMethod(this.loadMethod, 'ZxMigInit');
  this.registerModifyMethod(this.modifyMethod, 'ZxMigInit');

  this._helpURL = ZxLinks.HELP_MIG_MigrationTool;
  this.attrs[ZxModule.KEY_ENABLED_ON_STARTUP] = true;
  this.attrs[ZxModule.KEY_RUNNING] = true;
}

ZxMig.prototype = new ZxModule();
ZxMig.prototype.constructor = ZxMig;
ZxMig.MOD_ID = 'ZxMig';

ZxMig.KEY_COSLIST_ENABLED = 'cosList_Enabled';
ZxMig.KEY_COSLIST_DISABLED = 'cosList_Disabled';

ZxMig.KEY_UPDATE_INFO = 'updateinfo';
ZxMig.KEY_UPDATE_AVAILABLE = 'availableVersion';
ZxMig.KEY_UPDATE_CURRENT_VERSION = 'currentVersion';
ZxMig.KEY_UPDATE_LAST_UPDATE = 'lastUpdate';
ZxMig.KEY_UPDATE_CURRENT_COMMIT = 'currentCommit';
ZxMig.KEY_UPDATE_AVAILABLE_COMMIT = 'availableCommit';

ZxMig.KEY_PROPERTIES = 'properties';
ZxMig.KEY_NOTIFY_UPDATES = 'ZxCore_NotifyUpdates';
ZxMig.KEY_NOTIFICATION_LEVEL = 'ZxCore_NTLevel';
ZxMig.KEY_LOG_LEVEL = 'ZxCore_LogLevel';
ZxMig.KEY_NOTIFY_UPDATES = 'ZxCore_NotifyUpdates';
ZxMig.KEY_NOTIFICATION_TO = 'ZxCore_NTMail';

ZxMig.LogLevels = [
  {value: 0, label: zxMsg('LBL_LVL_debug')},
  {value: 1, label: zxMsg('LBL_LVL_information')},
  {value: 2, label: zxMsg('LBL_LVL_warning')},
  {value: 3, label: zxMsg('LBL_LVL_error')},
  {value: 4, label: zxMsg('LBL_LVL_critical')}
];

ZxMig.prototype.loadMethod = function(by, val) {
  this.attrs[ZxMig.KEY_UPDATE_INFO] = {};
  this.attrs[ZxMig.KEY_PROPERTIES] = {};
  this.attrs[ZxMig.KEY_COSLIST_ENABLED] = [];
  this.attrs[ZxMig.KEY_COSLIST_DISABLED] = [];
  this.attrs[ZxModule.KEY_ENABLED_ON_STARTUP] = true;
  this.attrs[ZxModule.KEY_RUNNING] = true;

  ZxAdminRequest.sendRequest(
    this.getServerName(),
    'ZxCore',
    'getPublicProperties'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var response = resp.response,
          controller = this.getCurrentController();

        this.attrs[ZxMig.KEY_PROPERTIES][ZxMig.KEY_NOTIFICATION_TO] =
          response[ZxMig.KEY_PROPERTIES][ZxMig.KEY_NOTIFICATION_TO];
        this.attrs[ZxMig.KEY_PROPERTIES][ZxMig.KEY_NOTIFICATION_LEVEL] =
          response[ZxMig.KEY_PROPERTIES][ZxMig.KEY_NOTIFICATION_LEVEL];
        this.attrs[ZxMig.KEY_PROPERTIES][ZxMig.KEY_LOG_LEVEL] =
          response[ZxMig.KEY_PROPERTIES][ZxMig.KEY_LOG_LEVEL];
        this.attrs[ZxMig.KEY_PROPERTIES][ZxMig.KEY_NOTIFY_UPDATES] =
          (response[ZxMig.KEY_PROPERTIES][ZxMig.KEY_NOTIFY_UPDATES] === 'true');

        controller.setInstanceValue(
          ZxMig.KEY_PROPERTIES + '/' + ZxMig.KEY_NOTIFICATION_TO,
          response[ZxMig.KEY_PROPERTIES][ZxMig.KEY_NOTIFICATION_TO]
        );
        controller.setInstanceValue(
          ZxMig.KEY_PROPERTIES + '/' + ZxMig.KEY_NOTIFICATION_LEVEL,
          response[ZxMig.KEY_PROPERTIES][ZxMig.KEY_NOTIFICATION_LEVEL]
        );
        controller.setInstanceValue(
          ZxMig.KEY_PROPERTIES + '/' + ZxMig.KEY_LOG_LEVEL,
          response[ZxMig.KEY_PROPERTIES][ZxMig.KEY_LOG_LEVEL]
        );
        controller.setInstanceValue(
          ZxMig.KEY_PROPERTIES + '/' + ZxMig.KEY_NOTIFY_UPDATES,
          (response[ZxMig.KEY_PROPERTIES][ZxMig.KEY_NOTIFY_UPDATES] === 'true')
        );
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );

  this.attrs[ZxMig.KEY_UPDATE_INFO] = {};

  ZxAdminRequest.sendRequest(
    this.getServerName(),
    'ZxCore',
    'getUpdateInfo'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
         var response = resp.response,
          controller = this.getCurrentController();

        this.attrs[ZxMig.KEY_UPDATE_INFO][ZxMig.KEY_UPDATE_AVAILABLE] = response[ZxMig.KEY_UPDATE_AVAILABLE];
        this.attrs[ZxMig.KEY_UPDATE_INFO][ZxMig.KEY_UPDATE_CURRENT_VERSION] =
          response[ZxMig.KEY_UPDATE_CURRENT_VERSION];
        this.attrs[ZxMig.KEY_UPDATE_INFO][ZxMig.KEY_UPDATE_LAST_UPDATE] = ZxUtils.computeWordyDateStr(
          new Date(),
          Number(response[ZxMig.KEY_UPDATE_LAST_UPDATE])
        );

        controller.setInstanceValue(
          ZxMig.KEY_UPDATE_INFO + '/' + ZxMig.KEY_UPDATE_AVAILABLE,
          response[ZxMig.KEY_UPDATE_AVAILABLE]
        );
        controller.setInstanceValue(
          ZxMig.KEY_UPDATE_INFO + ZxMig.KEY_UPDATE_AVAILABLE_COMMIT,
          response.availableCommit
        );
        controller.setInstanceValue(
          ZxMig.KEY_UPDATE_INFO + '/' + ZxMig.KEY_UPDATE_CURRENT_VERSION,
          response[ZxMig.KEY_UPDATE_CURRENT_VERSION]
        );
        controller.setInstanceValue(
          ZxMig.KEY_UPDATE_INFO + '/' + ZxMig.KEY_UPDATE_LAST_UPDATE,
          ZxUtils.computeWordyDateStr(
            new Date(),
            Number(response[ZxMig.KEY_UPDATE_LAST_UPDATE])
          )
        );
        controller.setInstanceValue(
          ZxMig.KEY_UPDATE_INFO + ZxMig.KEY_UPDATE_CURRENT_COMMIT,
          response.currentCommit
        );
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );

  ZxAdminRequest.sendRequest(
    this.getCurrentServer(),
    'ZxBackup',
    'getCOSBackupStatus'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        var server = this.getCurrentServer(),
          controller = this.getCurrentController(),
          response,
          tmpName,
          tmpListEnabled = [],
          tmpListDisabled = [];

        response = resp.response;
        for (tmpName in response.cosList) {
          if (!response.cosList.hasOwnProperty(tmpName)) { continue; }

          if (response.cosList[tmpName] === 'Enabled') {
            tmpListEnabled.push(tmpName);
          } else {
            tmpListDisabled.push(tmpName);
          }
        }

        tmpListEnabled.sort();
        tmpListDisabled.sort();

        this.attrs[ZxMig.KEY_COSLIST_ENABLED] = tmpListEnabled;
        this.attrs[ZxMig.KEY_COSLIST_DISABLED] = tmpListDisabled;

        controller.setInstanceValue(
          ZxMig.KEY_COSLIST_ENABLED,
          tmpListEnabled
        );
        controller.setInstanceValue(
          ZxMig.KEY_COSLIST_DISABLED,
          tmpListDisabled
        );
      },
      this
    ),
    this._getDefaultCatchErrorFcn()
  );

  ZeXtras.getOperationsManager().requestUpdate();

  var controller = this.getCurrentController();
  if (controller) {
    controller.setInstanceValue(
      'attrs/' + ZxModule.KEY_ENABLED_ON_STARTUP,
      true
    );
    controller.setInstanceValue(
      'attrs/' + ZxModule.KEY_RUNNING,
      true
    );
  }
};

/**
 * Enable or disable the export for a COS selected in the srcList.
 * @param {XForm} form
 * @param {DwtEvent} ev
 * @param {string} srcList
 * @param {string} destList
 * @see ZxBackup.doEnableDisableBackupForCOS
 */
ZxMig.doEnableDisableExportForCOS = function(form, ev, srcList, destList) {
  var enable = true,
    item = ZeXtras.getModuleItem(ZxMig, form.getInstance().id),
    cosList,
    selection = [];

  if (srcList === ZxMig.KEY_COSLIST_ENABLED && destList === ZxMig.KEY_COSLIST_DISABLED) {
    enable = false;
  } else if (srcList === ZxMig.KEY_COSLIST_DISABLED && destList === ZxMig.KEY_COSLIST_ENABLED) {
    enable = true;
  } else {
    ZxUtils.log({src: srcList, dst: destList}, 'Invalid parameters');
    return;
  }

  cosList = form.getItemsById(srcList)[0];
  selection = cosList.getSelection();
  if (selection.length === 0) {
    return;
  }

  item._doEnableDisableExportForCOS(selection, enable);
};

/**
 * real function to handle the enable/disable export for a list of COS(s).
 * @param {string[]} cosList
 * @param {boolean} enable
 */
ZxMig.prototype._doEnableDisableExportForCOS = function(cosList, enable) {
  var i = 0;

  for (i = 0; i < cosList.length; i = i + 1) {
    ZxAdminRequest.sendRequest(
      this.getServerName(),
      'ZxBackup',
      'doEnableDisableCOS',
      {
        COSName: cosList[i],
        enableCOS: enable ? 'enable' : 'disable'
      }
    ).then(
      AjxCallback.simpleClosure(
        this.fireLoadMethods,
        this
      ),
      ZxUtils.catchError
    );
  }

};

ZxMig.doStopAllOperations = function() {
  var item = ZeXtras.getModuleItem(ZxMig, this.getForm().getInstance().id),
    msgDialog = new DwtMessageDialog({
      buttons: [DwtDialog.YES_BUTTON, DwtDialog.NO_BUTTON],
      title: '',
      parent: ZaApp.getInstance().getCurrentController().getShell()
    });

  msgDialog.setMessage(
    zxMsg('MSG_MIG_Would_you_stop_operations'),
    DwtMessageDialog.WARNING_STYLE,
    ZxUtils.getTitleWithIcon('ZxMig', 'ZxMig')
  );
  msgDialog.registerCallback(
    DwtDialog.YES_BUTTON,
    AjxCallback.simpleClosure(
      function(dialog) {

        ZxAdminRequest.sendRequest(
          this.getServerName(),
          'ZxBackup',
          'doStopAllOperations'
        ).then(
          AjxCallback.simpleClosure(
            function(resp) {
              this.getCurrentController().popupMsgDialog(zxMsg('MSG_HSM_Termination_signal_sent'));
            },
            this
          ),
          ZxUtils.catchError
        );

        dialog.popdown();
      },
      item,
      msgDialog
    )
  );
  msgDialog.popup();
};

/**
 * Start the Export Wizard.
 * @param {DwtEvent} ev
 */
ZxMig.doStartExportWizard = function(ev) {
  var item = ZeXtras.getModuleItem(ZxMig, this.getForm().getInstance().id),
    opObj;

  try {
    ZaApp.getInstance().dialogs['zxExportWizard'] = new ZxExportWizard(item);

    opObj = new ZxExportOp(item);
    opObj.init();
    ZaApp.getInstance().dialogs['zxExportWizard'].setObject(opObj);
    ZaApp.getInstance().dialogs['zxExportWizard'].popup();

  } catch (ex) {
    ZxUtils.log(ex);
  }
};

/**
 * Check if new updates are available.
 * @this ZxCore
 */
ZxMig.doCheckUpdate = function() {
  var controller = ZaApp.getInstance().getCurrentController(),
    item = ZeXtras.getModuleItem(ZxMig, this.getForm().getInstance().id),

    loadingDialog = new ZxCheckUpdateDialog(
      controller.getShell(),
      item
    );
  loadingDialog.popup();
};
